package lib.turtle;

typedef Block = Bool;

abstract Structure(Array<Layer>) from Array<Layer> {
	public inline function getLayer(i:Int):Layer {
		return this[i];
	}

	public inline function height():Int {
		return this.length;
	}

	public function rotate():Structure {
		return [for (i in 0...height()) this[i].rotate()];
	}

	public function mirrorX():Structure {
		return [for (i in 0...height()) this[i].mirrorX()];
	}

	public function mirrorY():Structure {
		return [for (i in 0...height()) this[i].mirrorY()];
	}
}

abstract Layer(Array<Array<Block>>) from Array<Array<Block>> {
	public inline function get(x:Int, y:Int):Block {
		return this[y][x];
	}

	public inline function width():Int {
		return this[0].length;
	}

	public inline function height():Int {
		return this.length;
	}

	public function getTextRepresentation():String {
		var str = "";
		for (y in 0...height()) {
			for (x in 0...width()) {
				if (get(x, y)) {
					str += "X";
				} else {
					str += " ";
				}
			}
			str += "\n";
		}
		return str;
	}

	public function rotate():Layer {
		// Transpose the matrix (swap rows and columns)
		var transposedMatrix:Array<Array<Block>> = [];
		for (columnIndex in 0...width()) {
			var newRow:Array<Block> = [];
			for (rowIndex in 0...height()) {
				newRow.push(get(columnIndex, rowIndex));
			}
			transposedMatrix.push(newRow);
		}

		// Reverse the order of the columns
		var rotatedMatrix:Array<Array<Block>> = [];
		for (row in transposedMatrix) {
			var rev = row.copy();
			rev.reverse();
			rotatedMatrix.push(rev);
		}

		return rotatedMatrix;
	}

	public function mirrorX():Layer {
		var rtn = [];

		for (y in 0...height()) {
			var rev = this[y].copy();
			rev.reverse();
			rtn.push(rev);
		}

		return rtn;
	}

	public function mirrorY():Layer {
		var rtn = [for (_ in 0...height()) []];

		for (y in 0...height()) {
			for (x in 0...width()) {
				rtn[height() - 1 - y][x] = get(x, y);
			}
		}

		return rtn;
	}
}
